﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;

namespace Jumpy {
  class Player {
    enum Facing { Left, Right };
    enum Pose { Walking, Standing, Crouching, Stretching, SwordSwing, Jumping };
    enum AirState { Jumping, Ground };

    private Texture2D texture;
    private const int spriteSize = 48;
    private const int spriteWidth = 7;
    private const int moveSpeed = 200;
    private const int jumpSpeed = 600;
    private const int gravity = 2000;
    private const int groundLevel = Camera.Height - spriteSize / 2;

    private Point position = new Point(Camera.Width / 2, groundLevel);
    private Facing facing = Facing.Right;
    private Pose pose = Pose.Standing;
    private AirState airState = AirState.Ground;
    private double swordSwingTime = 0;
    private double jumpTime = 0;
    private double ySpeed = 0;

    public Player(Texture2D texture) {
      this.texture = texture;
    }

    // TODO: refactor input to have a virtual "which directions & buttons were being pressed"
    // instead of complicated if-statements in this function.
    public void Update(GameTime time, History<GamePadState> gamePad) {
      if (gamePad[0].IsButtonDown(Buttons.A) && airState == AirState.Ground) {
        pose = Pose.Jumping;
        airState = AirState.Jumping;
        jumpTime = 0.5;
        ySpeed = -jumpSpeed;
        return;
      }

      if (gamePad[0].IsButtonDown(Buttons.X) && gamePad[1].IsButtonUp(Buttons.X)
          && swordSwingTime <= 0) {
        pose = Pose.SwordSwing;
        swordSwingTime = 0.3;
        return;
      }

      Vector2 leftStick = gamePad[0].ThumbSticks.Left;
      if (gamePad[0].IsButtonDown(Buttons.DPadLeft) || leftStick.X < -0.5) {
        facing = Facing.Left;
        pose = Pose.Walking;
        position.X -= (int) (moveSpeed * time.ElapsedGameTime.TotalSeconds);
      } else if (gamePad[0].IsButtonDown(Buttons.DPadRight) || leftStick.X > 0.5) {
        facing = Facing.Right;
        pose = Pose.Walking;
        position.X += (int) (moveSpeed * time.ElapsedGameTime.TotalSeconds);
      } else if (gamePad[0].IsButtonDown(Buttons.DPadDown) || leftStick.Y < -0.5) {
        pose = Pose.Crouching;
      } else if (gamePad[0].IsButtonDown(Buttons.DPadUp) || leftStick.Y > 0.5) {
        pose = Pose.Stretching;
      } else {
        pose = Pose.Standing;
      }

      if (jumpTime > 0) {
        jumpTime -= time.ElapsedGameTime.TotalSeconds;
      }
      if (swordSwingTime > 0) {
        swordSwingTime -= time.ElapsedGameTime.TotalSeconds;
        pose = Pose.SwordSwing;
      }

      if (airState == AirState.Jumping) {
        position.Y += (int) (ySpeed * time.ElapsedGameTime.TotalSeconds);
        ySpeed += gravity * (float) time.ElapsedGameTime.TotalSeconds;
        if (position.Y > groundLevel) {
          position.Y = groundLevel;
          ySpeed = 0;
          airState = AirState.Ground;
        }
      }
      if (airState == AirState.Jumping && pose != Pose.SwordSwing) {
        pose = Pose.Jumping;
      }
   
      position.X = Math.Min(Math.Max(position.X, 0 + spriteWidth), Camera.Width - spriteWidth);
    }

    private Point spritePosition(Pose pose, GameTime time) {
      int frameNum = (time.TotalGameTime.Milliseconds / 125) % 4;
      if (frameNum == 3) {
        frameNum = 1;
      }

      switch (pose) {
        case Pose.Walking:
          return new Point(spriteSize * frameNum + spriteSize * 6, 0);
        case Pose.Crouching:
          return new Point(spriteSize * 7, spriteSize * 2);
        case Pose.Stretching:
          return new Point(spriteSize * frameNum, spriteSize * 2);
        case Pose.Jumping:
          if (jumpTime > 0.25) {
            return new Point(spriteSize * 6, spriteSize);
          } else if (jumpTime > 0) {
            return new Point(spriteSize * 7, spriteSize);
          } else {
            return new Point(spriteSize * 8, spriteSize);
          }
        case Pose.SwordSwing:
          if (swordSwingTime > 0.2) {
            return new Point(spriteSize * 3, spriteSize * 0);
          } else if (swordSwingTime > 0.1) {
            return new Point(spriteSize * 4, spriteSize * 0);
          } else {
            return new Point(spriteSize * 5, spriteSize * 0);
          }
        case Pose.Standing:
        default:
          return new Point(spriteSize * 7, 0);
      }
    }

    public void Draw(GameTime time, SpriteBatch spriteBatch) {
      Point source = spritePosition(pose, time);
      Rectangle textureSource = new Rectangle(source.X, source.Y, spriteSize, spriteSize);
      Vector2 spriteCenter = new Vector2(spriteSize / 2, spriteSize / 2);
      SpriteEffects effect = facing == Facing.Right ?
        SpriteEffects.FlipHorizontally : SpriteEffects.None;
      Vector2 drawPos = new Vector2(position.X, position.Y);
      spriteBatch.Draw(texture, drawPos, textureSource, Color.White, 0f, spriteCenter,
        Vector2.One, effect, 0f);
    }
  }
}
